@@ -71,8 +71,8 @@ class BaseSgmlLinkExtractor(FixedSGMLParser):
         self.current_link = None
 
     def handle_data(self, data):
-        if self.current_link and not self.current_link.text:
-            self.current_link.text = data.strip()
+        if self.current_link:
+            self.current_link.text = self.current_link.text + data.strip()
 
     def matches(self, url):
         """This extractor matches with any url, since
