@@ -20,10 +20,10 @@ from scrapy.xlib.pydispatch import dispatcher
 from scrapy import log
 from scrapy.stats import stats
 from scrapy.utils.misc import md5sum
-from scrapy.core import signals
-from scrapy.core.exceptions import DropItem, NotConfigured, IgnoreRequest
+from scrapy.http import Request
+from scrapy import signals
+from scrapy.exceptions import DropItem, NotConfigured, IgnoreRequest
 from scrapy.contrib.pipeline.media import MediaPipeline
-from scrapy.conf import settings
 
 
 class NoimagesDrop(DropItem):
@@ -77,8 +77,14 @@ class FSImagesStore(object):
 
 class S3ImagesStore(object):
 
-    AWS_ACCESS_KEY_ID = settings['AWS_ACCESS_KEY_ID']
-    AWS_SECRET_ACCESS_KEY = settings['AWS_SECRET_ACCESS_KEY']
+    AWS_ACCESS_KEY_ID = None
+    AWS_SECRET_ACCESS_KEY = None
+
+    POLICY = 'public-read'
+    HEADERS = {
+            'Cache-Control': 'max-age=172800',
+            'Content-Type': 'image/jpeg',
+            }
 
     def __init__(self, uri):
         assert uri.startswith('s3://')
@@ -107,15 +113,14 @@ class S3ImagesStore(object):
     def persist_image(self, key, image, buf, info):
         """Upload image to S3 storage"""
         width, height = image.size
-        headers = {'Cache-Control': 'max-age=172800'} # 2 days of cache
         b = self._get_boto_bucket()
         key_name = '%s%s' % (self.prefix, key)
         k = b.new_key(key_name)
         k.set_metadata('width', str(width))
         k.set_metadata('height', str(height))
         buf.seek(0)
-        return threads.deferToThread(k.set_contents_from_file, buf, headers, \
-            policy='public-read')
+        return threads.deferToThread(k.set_contents_from_file, buf, \
+                headers=self.HEADERS, policy=self.POLICY)
 
 
 class ImagesPipeline(MediaPipeline):
@@ -138,22 +143,33 @@ class ImagesPipeline(MediaPipeline):
     """
 
     MEDIA_NAME = 'image'
-    MIN_WIDTH = settings.getint('IMAGES_MIN_WIDTH', 0)
-    MIN_HEIGHT = settings.getint('IMAGES_MIN_HEIGHT', 0)
-    EXPIRES = settings.getint('IMAGES_EXPIRES', 90)
-    THUMBS = settings.get('IMAGES_THUMBS', {})
+    MIN_WIDTH = 0
+    MIN_HEIGHT = 0
+    EXPIRES = 90
+    THUMBS = {}
     STORE_SCHEMES = {
             '': FSImagesStore,
             'file': FSImagesStore,
             's3': S3ImagesStore,
             }
 
-    def __init__(self):
+    def __init__(self, store_uri):
+        self.store = self._get_store(store_uri)
+        super(ImagesPipeline, self).__init__()
+
+    @classmethod
+    def from_settings(cls, settings):
+        cls.MIN_WIDTH = settings.getint('IMAGES_MIN_WIDTH', 0)
+        cls.MIN_HEIGHT = settings.getint('IMAGES_MIN_HEIGHT', 0)
+        cls.EXPIRES = settings.getint('IMAGES_EXPIRES', 90)
+        cls.THUMBS = settings.get('IMAGES_THUMBS', {})
+        s3store = cls.STORE_SCHEMES['s3']
+        s3store.AWS_ACCESS_KEY_ID = settings['AWS_ACCESS_KEY_ID']
+        s3store.AWS_SECRET_ACCESS_KEY = settings['AWS_SECRET_ACCESS_KEY']
         store_uri = settings['IMAGES_STORE']
         if not store_uri:
             raise NotConfigured
-        self.store = self._get_store(store_uri)
-        super(ImagesPipeline, self).__init__()
+        return cls(store_uri)
 
     def _get_store(self, uri):
         if os.path.isabs(uri): # to support win32 paths like: C:\\some\dir
@@ -261,7 +277,11 @@ class ImagesPipeline(MediaPipeline):
         stats.inc_value('image_status_count/%s' % status, spider=spider)
 
     def convert_image(self, image, size=None):
-        if image.mode != 'RGB':
+        if image.format == 'PNG' and image.mode == 'RGBA':
+            background = Image.new('RGBA', image.size, (255, 255, 255))
+            background.paste(image, image)
+            image = background.convert('RGB')
+        elif image.mode != 'RGB':
             image = image.convert('RGB')
 
         if size:
@@ -283,3 +303,10 @@ class ImagesPipeline(MediaPipeline):
     def thumb_key(self, url, thumb_id):
         image_guid = hashlib.sha1(url).hexdigest()
         return 'thumbs/%s/%s.jpg' % (thumb_id, image_guid)
+
+    def get_media_requests(self, item, info):
+        return [Request(x) for x in item.get('image_urls', [])]
+
+    def item_completed(self, results, item, info):
+        item['images'] = [x for ok, x in results if ok]
+        return item
