@@ -1,8 +1,9 @@
+import os
 import unittest
+from scrapy.spider import BaseSpider
 from scrapy.utils.url import url_is_from_any_domain, safe_url_string, safe_download_url, \
     url_query_parameter, add_or_replace_parameter, url_query_cleaner, canonicalize_url, \
-    urljoin_rfc
-
+    urljoin_rfc, url_is_from_spider, file_uri_to_path, path_to_file_uri, any_to_uri
 
 class UrlUtilsTest(unittest.TestCase):
 
@@ -19,6 +20,41 @@ class UrlUtilsTest(unittest.TestCase):
         self.assertFalse(url_is_from_any_domain(url, ['testdomain.com']))
         self.assertFalse(url_is_from_any_domain(url+'.testdomain.com', ['testdomain.com']))
 
+    def test_url_is_from_spider(self):
+        spider = BaseSpider(name='example.com')
+        self.assertTrue(url_is_from_spider('http://www.example.com/some/page.html', spider))
+        self.assertTrue(url_is_from_spider('http://sub.example.com/some/page.html', spider))
+        self.assertFalse(url_is_from_spider('http://www.example.org/some/page.html', spider))
+        self.assertFalse(url_is_from_spider('http://www.example.net/some/page.html', spider))
+
+    def test_url_is_from_spider_class_attributes(self):
+        class MySpider(BaseSpider):
+            name = 'example.com'
+        self.assertTrue(url_is_from_spider('http://www.example.com/some/page.html', MySpider))
+        self.assertTrue(url_is_from_spider('http://sub.example.com/some/page.html', MySpider))
+        self.assertFalse(url_is_from_spider('http://www.example.org/some/page.html', MySpider))
+        self.assertFalse(url_is_from_spider('http://www.example.net/some/page.html', MySpider))
+
+    def test_url_is_from_spider_with_allowed_domains(self):
+        spider = BaseSpider(name='example.com', allowed_domains=['example.org', 'example.net'])
+        self.assertTrue(url_is_from_spider('http://www.example.com/some/page.html', spider))
+        self.assertTrue(url_is_from_spider('http://sub.example.com/some/page.html', spider))
+        self.assertTrue(url_is_from_spider('http://example.com/some/page.html', spider))
+        self.assertTrue(url_is_from_spider('http://www.example.org/some/page.html', spider))
+        self.assertTrue(url_is_from_spider('http://www.example.net/some/page.html', spider))
+        self.assertFalse(url_is_from_spider('http://www.example.us/some/page.html', spider))
+
+    def test_url_is_from_spider_with_allowed_domains_class_attributes(self):
+        class MySpider(BaseSpider):
+            name = 'example.com'
+            allowed_domains = ['example.org', 'example.net']
+        self.assertTrue(url_is_from_spider('http://www.example.com/some/page.html', MySpider))
+        self.assertTrue(url_is_from_spider('http://sub.example.com/some/page.html', MySpider))
+        self.assertTrue(url_is_from_spider('http://example.com/some/page.html', MySpider))
+        self.assertTrue(url_is_from_spider('http://www.example.org/some/page.html', MySpider))
+        self.assertTrue(url_is_from_spider('http://www.example.net/some/page.html', MySpider))
+        self.assertFalse(url_is_from_spider('http://www.example.us/some/page.html', MySpider))
+
     def test_urljoin_rfc(self):
         self.assertEqual(urljoin_rfc('http://example.com/some/path', 'newpath/test'),
                                      'http://example.com/some/newpath/test')
@@ -238,6 +274,51 @@ class UrlUtilsTest(unittest.TestCase):
         self.assertEqual(canonicalize_url(u'http://www.example.com/caf%E9-con-leche.htm'),
                                            'http://www.example.com/caf%E9-con-leche.htm')
 
+    def test_path_to_file_uri(self):
+        if os.name == 'nt':
+            self.assertEqual(path_to_file_uri("C:\\windows\clock.avi"),
+                             "file:///C:/windows/clock.avi")
+        else:
+            self.assertEqual(path_to_file_uri("/some/path.txt"),
+                             "file:///some/path.txt")
+
+        fn = "test.txt"
+        x = path_to_file_uri(fn)
+        self.assert_(x.startswith('file:///'))
+        self.assertEqual(file_uri_to_path(x).lower(), os.path.abspath(fn).lower())
+
+    def test_file_uri_to_path(self):
+        if os.name == 'nt':
+            self.assertEqual(file_uri_to_path("file:///C:/windows/clock.avi"),
+                             "C:\\windows\clock.avi")
+            uri = "file:///C:/windows/clock.avi"
+            uri2 = path_to_file_uri(file_uri_to_path(uri))
+            self.assertEqual(uri, uri2)
+        else:
+            self.assertEqual(file_uri_to_path("file:///path/to/test.txt"),
+                             "/path/to/test.txt")
+            self.assertEqual(file_uri_to_path("/path/to/test.txt"),
+                             "/path/to/test.txt")
+            uri = "file:///path/to/test.txt"
+            uri2 = path_to_file_uri(file_uri_to_path(uri))
+            self.assertEqual(uri, uri2)
+
+        self.assertEqual(file_uri_to_path("test.txt"),
+                         "test.txt")
+
+    def test_any_to_uri(self):
+        if os.name == 'nt':
+            self.assertEqual(any_to_uri("C:\\windows\clock.avi"),
+                             "file:///C:/windows/clock.avi")
+        else:
+            self.assertEqual(any_to_uri("/some/path.txt"),
+                             "file:///some/path.txt")
+        self.assertEqual(any_to_uri("file:///some/path.txt"),
+                         "file:///some/path.txt")
+        self.assertEqual(any_to_uri("http://www.example.com/some/path.txt"),
+                         "http://www.example.com/some/path.txt")
+
+
 if __name__ == "__main__":
     unittest.main()
 
